@@ -51,7 +51,7 @@
 
 /* Maximum digits in a number (i.e. number of digits in score, */
 /* number of blocks, etc. should not exceed this value */
-#define MAXDIGITS 5
+#define MAXDIGITS 11
 
 /* Number of levels in the game */
 #define MINLEVEL	1
@@ -69,6 +69,13 @@
 /* This calculates the real (displayed) value of the score */
 #define GETSCORE(score) ((score) / SCOREFACTOR)
 
+/* Remove extra input after fgets if needed */
+#define ENDFGETS(buffer) \
+   if (!strchr(buffer, '\n')) { \
+      int ch; \
+      while (((ch = getchar()) != EOF) && (ch != '\n')); \
+   }
+
 static bool shownext;
 static bool dottedlines;
 static int level = MINLEVEL - 1,shapecount[NUMSHAPES];
@@ -329,7 +336,9 @@ static void getname (char *name)
    fprintf (stderr,"Congratulations! You have a new high score.\n");
    fprintf (stderr,"Enter your name [%s]: ",pw != NULL ? pw->pw_name : "");
 
-   fgets (name,NAMELEN - 1,stdin);
+   memset(name, 0, NAMELEN);
+   fgets(name, NAMELEN - 1, stdin);
+   ENDFGETS(name)
    name[strlen (name) - 1] = '\0';
 
    if (!strlen (name) && pw != NULL)
@@ -553,7 +562,12 @@ static void choose_level ()
    do
 	 {
 		fprintf (stderr,"Choose a level to start [%d-%d]: ",MINLEVEL,MAXLEVEL);
-		fgets (buf,NAMELEN - 1,stdin);
+		if (NULL == fgets(buf, NAMELEN - 1, stdin)) {
+			fprintf(stderr, "\nGoodbye!\n");
+			level = -1;
+			break;
+		}
+		ENDFGETS(buf)
 		buf[strlen (buf) - 1] = '\0';
 	 }
    while (!str2int (&level,buf) || level < MINLEVEL || level > MAXLEVEL);
@@ -576,6 +590,7 @@ int main (int argc,char *argv[])
    shapecount[engine.curshape]++;
    parse_options (argc,argv);				/* must be called after initializing variables */
    if (level < MINLEVEL) choose_level ();
+   if (level < 0) exit (EXIT_SUCCESS);
    io_init ();
    drawbackground ();
    in_timeout (DELAY);
